#ifndef evoltrunc_h
#define evoltrunc_h

#include "hell-x.hh"
#include "pmom.h"
#include "constants.h"
#include "resconst.h"
#include "resint.h"
#include "pdfevol.h"

#include <complex>

using namespace std;
using namespace pmom;
using namespace constants;
using namespace resconst;

typedef HELLx::sqmatrix<complex<double>> cmatrix;

namespace evoltrunc
{
  extern int dim;
  extern complex <double> *gli;
  extern complex <double> *vai;
  extern complex <double> *m3i;
  extern complex <double> *m8i;
  extern complex <double> *m15i;
  extern complex <double> *m24i;
  extern complex <double> *sgi;
  extern complex <double> *p3i;
  extern complex <double> *p8i;
  extern complex <double> *p15i;
  extern complex <double> *p24i;
  extern complex <double> *cxn;
#pragma omp threadprivate(gli,vai,m3i,m8i,m15i,m24i,sgi,p3i,p8i,p15i,p24i,cxn)

  extern complex <double> *gli_muf;
  extern complex <double> *vai_muf;
  extern complex <double> *m3i_muf;
  extern complex <double> *m8i_muf;
  extern complex <double> *m15i_muf;
  extern complex <double> *m24i_muf;
  extern complex <double> *sgi_muf;
  extern complex <double> *p3i_muf;
  extern complex <double> *p8i_muf;
  extern complex <double> *p15i_muf;
  extern complex <double> *p24i_muf;
#pragma omp threadprivate(gli_muf,vai_muf,m3i_muf,m8i_muf,m15i_muf,m24i_muf,sgi_muf,p3i_muf,p8i_muf,p15i_muf,p24i_muf)
  
  extern complex <double> *rp;
  extern complex <double> *rm;
  extern cmatrix *Ep;
  extern cmatrix *Em;
  extern cmatrix *U1;
  extern cmatrix *U2;
  extern cmatrix *U3;
  extern cmatrix *U;
#pragma omp threadprivate(rp,rm,Ep,Em,U1,U2,U3,U)

  extern complex <double> *R0NS;
  extern complex <double> *U1NSp;
  extern complex <double> *U1NSm;
  extern complex <double> *U1NSv;
  extern complex <double> *U2NSp;
  extern complex <double> *U2NSm;
  extern complex <double> *U2NSv;
  extern complex <double> *U3NSp;
  extern complex <double> *U3NSm;
  extern complex <double> *U3NSv;
  extern complex <double> *UNSp;
  extern complex <double> *UNSm;
  extern complex <double> *UNSv;
#pragma omp threadprivate(R0NS,U1NSp,U1NSm,U1NSv,U2NSp,U2NSm,U2NSv,U3NSp,U3NSm,U3NSv,UNSp,UNSm,UNSv)
  
  extern const int maxord;

  inline int idx(int o, int i) {return o*2*dim+i;}

  extern const cmatrix I;
  extern const cmatrix N;

  extern double beta0;
  extern double beta1;
  extern double beta2;
  extern double beta3;

  extern double b1;
  extern double b2;
  extern double b3;

  extern double beta(double as, int o);
  
  //Singlet
  inline cmatrix gamma_LO   (int i) {return cmatrix (gamma1gg[i]/M_PI, gamma1gq[i]/M_PI, 2.*NF*gamma1qg[i]/M_PI, gamma1qq[i]/M_PI);}
  inline cmatrix gamma_NLO  (int i) {return cmatrix (gamma2gg[i]/pi2,  gamma2gq[i]/pi2,  2.*NF*gamma2qg[i]/pi2, (((gamma2qq[i]+(NF-1.)*gamma2qqp[i])+(gamma2qqb[i]+(NF-1.)*gamma2qqbp[i]))/pi2));}
  inline cmatrix gamma_NNLO (int i) {return cmatrix (gamma3gg[i]/pi3,  gamma3gq[i]/pi3,  2.*NF*gamma3qg[i]/pi3, (((gamma3qq[i]+(NF-1.)*gamma3qqp[i])+(gamma3qqb[i]+(NF-1.)*gamma3qqbp[i]))/pi3));}
  inline cmatrix gamma_NNNLO(int i) {return cmatrix (gamma4gg[i]/pi4,  gamma4gq[i]/pi4,  2.*NF*gamma4qg[i]/pi4, (((gamma4qq[i]+(NF-1.)*gamma4qqp[i])+(gamma4qqb[i]+(NF-1.)*gamma4qqbp[i]))/pi4));}
  //inline cmatrix gammaLO_as   (double as, int i) {return ( as*gamma_LO(i) ) / ( -beta0*as*as );}
  //inline cmatrix gammaNLO_as  (double as, int i) {return ( as*gamma_LO(i) + as*as*gamma_NLO(i) ) / ( -beta0*as*as -beta1*as*as*as);}
  //inline cmatrix gammaNNLO_as (double as, int i) {return ( as*gamma_LO(i) + as*as*gamma_NLO(i) + as*as*as*gamma_NNLO(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as);}
  //inline cmatrix gammaNNNLO_as(double as, int i) {return ( as*gamma_LO(i) + as*as*gamma_NLO(i) + as*as*as*gamma_NNLO(i) + as*as*as*as*gamma_NNNLO(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as -beta3*as*as*as*as*as);}

  inline cmatrix gamma_LO_1   (int i) {return cmatrix (gamma1gg_1[i]/M_PI, gamma1gq_1[i]/M_PI, 2.*NF*gamma1qg_1[i]/M_PI, gamma1qq_1[i]/M_PI);}
  inline cmatrix gamma_NLO_1  (int i) {return cmatrix (gamma2gg_1[i]/pi2,  gamma2gq_1[i]/pi2,  2.*NF*gamma2qg_1[i]/pi2, (((gamma2qq_1[i]+(NF-1.)*gamma2qqp_1[i])+(gamma2qqb_1[i]+(NF-1.)*gamma2qqbp_1[i]))/pi2));}
  inline cmatrix gamma_NNLO_1 (int i) {return cmatrix (gamma3gg_1[i]/pi3,  gamma3gq_1[i]/pi3,  2.*NF*gamma3qg_1[i]/pi3, (((gamma3qq_1[i]+(NF-1.)*gamma3qqp_1[i])+(gamma3qqb_1[i]+(NF-1.)*gamma3qqbp_1[i]))/pi3));}
  inline cmatrix gamma_NNNLO_1(int i) {return cmatrix (gamma4gg_1[i]/pi4,  gamma4gq_1[i]/pi4,  2.*NF*gamma4qg_1[i]/pi4, (((gamma4qq_1[i]+(NF-1.)*gamma4qqp_1[i])+(gamma4qqb_1[i]+(NF-1.)*gamma4qqbp_1[i]))/pi4));}
  //inline cmatrix gammaLO_as_1   (double as, int i) {return ( as*gamma_LO_1(i) ) / ( -beta0*as*as );}
  //inline cmatrix gammaNLO_as_1  (double as, int i) {return ( as*gamma_LO_1(i) + as*as*gamma_NLO_1(i) ) / ( -beta0*as*as -beta1*as*as*as);}
  //inline cmatrix gammaNNLO_as_1 (double as, int i) {return ( as*gamma_LO_1(i) + as*as*gamma_NLO_1(i) + as*as*as*gamma_NNLO_1(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as);}
  //inline cmatrix gammaNNNLO_as_1(double as, int i) {return ( as*gamma_LO_1(i) + as*as*gamma_NLO_1(i) + as*as*as*gamma_NNLO_1(i) + as*as*as*as*gamma_NNNLO_1(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as -beta3*as*as*as*as*as);}
  inline cmatrix gamma_LO_2   (int i) {return cmatrix (gamma1gg_2[i]/M_PI, gamma1gq_2[i]/M_PI, 2.*NF*gamma1qg_2[i]/M_PI, gamma1qq_2[i]/M_PI);}
  inline cmatrix gamma_NLO_2  (int i) {return cmatrix (gamma2gg_2[i]/pi2,  gamma2gq_2[i]/pi2,  2.*NF*gamma2qg_2[i]/pi2, (((gamma2qq_2[i]+(NF-1.)*gamma2qqp_2[i])+(gamma2qqb_2[i]+(NF-1.)*gamma2qqbp_2[i]))/pi2));}
  inline cmatrix gamma_NNLO_2 (int i) {return cmatrix (gamma3gg_2[i]/pi3,  gamma3gq_2[i]/pi3,  2.*NF*gamma3qg_2[i]/pi3, (((gamma3qq_2[i]+(NF-1.)*gamma3qqp_2[i])+(gamma3qqb_2[i]+(NF-1.)*gamma3qqbp_2[i]))/pi3));}
  inline cmatrix gamma_NNNLO_2(int i) {return cmatrix (gamma4gg_2[i]/pi4,  gamma4gq_2[i]/pi4,  2.*NF*gamma4qg_2[i]/pi4, (((gamma4qq_2[i]+(NF-1.)*gamma4qqp_2[i])+(gamma4qqb_2[i]+(NF-1.)*gamma4qqbp_2[i]))/pi4));}
  //inline cmatrix gammaLO_as_2  (double as, int i) {return ( as*gamma_LO_2(i) ) / ( -beta0*as*as );}
  //inline cmatrix gammaNLO_as_2 (double as, int i) {return ( as*gamma_LO_2(i) + as*as*gamma_NLO_2(i) ) / ( -beta0*as*as -beta1*as*as*as);}
  //inline cmatrix gammaNNLO_as_2(double as, int i) {return ( as*gamma_LO_2(i) + as*as*gamma_NLO_2(i) + as*as*as*gamma_NNLO_2(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as);}
  //inline cmatrix gammaNNNLO_as_2(double as, int i) {return ( as*gamma_LO_2(i) + as*as*gamma_NLO_2(i) + as*as*as*gamma_NNLO_2(i) + as*as*as*as*gamma_NNNLO_2(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as -beta3*as*as*as*as*as);}

  //Non Singlet
  inline complex <double> gammaNSp_LO(int i) {return (gamma1qq[i] + gamma1qqb[i] - gamma1qqp[i] - gamma1qqbp[i])/M_PI;}
  inline complex <double> gammaNSm_LO(int i) {return (gamma1qq[i] - gamma1qqb[i] - gamma1qqp[i] + gamma1qqbp[i])/M_PI;}
  inline complex <double> gammaNSv_LO(int i) {return (((gamma1qq[i]+(NF-1.)*gamma1qqp[i])-(gamma1qqb[i]+(NF-1.)*gamma1qqbp[i]))/M_PI);}
  inline complex <double> gammaNSp_NLO(int i) {return (gamma2qq[i] + gamma2qqb[i] - gamma2qqp[i] - gamma2qqbp[i])/pi2;}
  inline complex <double> gammaNSm_NLO(int i) {return (gamma2qq[i] - gamma2qqb[i] - gamma2qqp[i] + gamma2qqbp[i])/pi2;}
  inline complex <double> gammaNSv_NLO(int i) {return (((gamma2qq[i]+(NF-1.)*gamma2qqp[i])-(gamma2qqb[i]+(NF-1.)*gamma2qqbp[i]))/pi2);}
  inline complex <double> gammaNSp_NNLO(int i) {return (gamma3qq[i] + gamma3qqb[i] - gamma3qqp[i] - gamma3qqbp[i])/pi3;}
  inline complex <double> gammaNSm_NNLO(int i) {return (gamma3qq[i] - gamma3qqb[i] - gamma3qqp[i] + gamma3qqbp[i])/pi3;}
  inline complex <double> gammaNSv_NNLO(int i) {return (((gamma3qq[i]+(NF-1.)*gamma3qqp[i])-(gamma3qqb[i]+(NF-1.)*gamma3qqbp[i]))/pi3);}
  inline complex <double> gammaNSp_NNNLO(int i) {return (gamma4qq[i] + gamma4qqb[i] - gamma4qqp[i] - gamma4qqbp[i])/pi4;}
  inline complex <double> gammaNSm_NNNLO(int i) {return (gamma4qq[i] - gamma4qqb[i] - gamma4qqp[i] + gamma4qqbp[i])/pi4;}
  inline complex <double> gammaNSv_NNNLO(int i) {return (((gamma4qq[i]+(NF-1.)*gamma4qqp[i])-(gamma4qqb[i]+(NF-1.)*gamma4qqbp[i]))/pi4);}
  inline complex <double> gammaNSp_LO_1(int i) {return (gamma1qq_1[i] + gamma1qqb_1[i] - gamma1qqp_1[i] - gamma1qqbp_1[i])/M_PI;}
  inline complex <double> gammaNSm_LO_1(int i) {return (gamma1qq_1[i] - gamma1qqb_1[i] - gamma1qqp_1[i] + gamma1qqbp_1[i])/M_PI;}
  inline complex <double> gammaNSv_LO_1(int i) {return (((gamma1qq_1[i]+(NF-1.)*gamma1qqp_1[i])-(gamma1qqb_1[i]+(NF-1.)*gamma1qqbp_1[i]))/M_PI);}
  inline complex <double> gammaNSp_NLO_1(int i) {return (gamma2qq_1[i] + gamma2qqb_1[i] - gamma2qqp_1[i] - gamma2qqbp_1[i])/pi2;}
  inline complex <double> gammaNSm_NLO_1(int i) {return (gamma2qq_1[i] - gamma2qqb_1[i] - gamma2qqp_1[i] + gamma2qqbp_1[i])/pi2;}
  inline complex <double> gammaNSv_NLO_1(int i) {return (((gamma2qq_1[i]+(NF-1.)*gamma2qqp_1[i])-(gamma2qqb_1[i]+(NF-1.)*gamma2qqbp_1[i]))/pi2);}
  inline complex <double> gammaNSp_NNLO_1(int i) {return (gamma3qq_1[i] + gamma3qqb_1[i] - gamma3qqp_1[i] - gamma3qqbp_1[i])/pi3;}
  inline complex <double> gammaNSm_NNLO_1(int i) {return (gamma3qq_1[i] - gamma3qqb_1[i] - gamma3qqp_1[i] + gamma3qqbp_1[i])/pi3;}
  inline complex <double> gammaNSv_NNLO_1(int i) {return (((gamma3qq_1[i]+(NF-1.)*gamma3qqp_1[i])-(gamma3qqb_1[i]+(NF-1.)*gamma3qqbp_1[i]))/pi3);}
  inline complex <double> gammaNSp_NNNLO_1(int i) {return (gamma4qq_1[i] + gamma4qqb_1[i] - gamma4qqp_1[i] - gamma4qqbp_1[i])/pi4;}
  inline complex <double> gammaNSm_NNNLO_1(int i) {return (gamma4qq_1[i] - gamma4qqb_1[i] - gamma4qqp_1[i] + gamma4qqbp_1[i])/pi4;}
  inline complex <double> gammaNSv_NNNLO_1(int i) {return (((gamma4qq_1[i]+(NF-1.)*gamma4qqp_1[i])-(gamma4qqb_1[i]+(NF-1.)*gamma4qqbp_1[i]))/pi4);}
  inline complex <double> gammaNSp_LO_2(int i) {return (gamma1qq_2[i] + gamma1qqb_2[i] - gamma1qqp_2[i] - gamma1qqbp_2[i])/M_PI;}
  inline complex <double> gammaNSm_LO_2(int i) {return (gamma1qq_2[i] - gamma1qqb_2[i] - gamma1qqp_2[i] + gamma1qqbp_2[i])/M_PI;}
  inline complex <double> gammaNSv_LO_2(int i) {return (((gamma1qq_2[i]+(NF-1.)*gamma1qqp_2[i])-(gamma1qqb_2[i]+(NF-1.)*gamma1qqbp_2[i]))/M_PI);}
  inline complex <double> gammaNSp_NLO_2(int i) {return (gamma2qq_2[i] + gamma2qqb_2[i] - gamma2qqp_2[i] - gamma2qqbp_2[i])/pi2;}
  inline complex <double> gammaNSm_NLO_2(int i) {return (gamma2qq_2[i] - gamma2qqb_2[i] - gamma2qqp_2[i] + gamma2qqbp_2[i])/pi2;}
  inline complex <double> gammaNSv_NLO_2(int i) {return (((gamma2qq_2[i]+(NF-1.)*gamma2qqp_2[i])-(gamma2qqb_2[i]+(NF-1.)*gamma2qqbp_2[i]))/pi2);}
  inline complex <double> gammaNSp_NNLO_2(int i) {return (gamma3qq_2[i] + gamma3qqb_2[i] - gamma3qqp_2[i] - gamma3qqbp_2[i])/pi3;}
  inline complex <double> gammaNSm_NNLO_2(int i) {return (gamma3qq_2[i] - gamma3qqb_2[i] - gamma3qqp_2[i] + gamma3qqbp_2[i])/pi3;}
  inline complex <double> gammaNSv_NNLO_2(int i) {return (((gamma3qq_2[i]+(NF-1.)*gamma3qqp_2[i])-(gamma3qqb_2[i]+(NF-1.)*gamma3qqbp_2[i]))/pi3);}
  inline complex <double> gammaNSp_NNNLO_2(int i) {return (gamma4qq_2[i] + gamma4qqb_2[i] - gamma4qqp_2[i] - gamma4qqbp_2[i])/pi4;}
  inline complex <double> gammaNSm_NNNLO_2(int i) {return (gamma4qq_2[i] - gamma4qqb_2[i] - gamma4qqp_2[i] + gamma4qqbp_2[i])/pi4;}
  inline complex <double> gammaNSv_NNNLO_2(int i) {return (((gamma4qq_2[i]+(NF-1.)*gamma4qqp_2[i])-(gamma4qqb_2[i]+(NF-1.)*gamma4qqbp_2[i]))/pi4);}

  inline complex <double> gammaLO_NSp_as (double as, int i) {return ( as*gammaNSp_LO(i) ) / ( -beta0*as*as );}
  inline complex <double> gammaLO_NSm_as (double as, int i) {return ( as*gammaNSm_LO(i) ) / ( -beta0*as*as );}
  inline complex <double> gammaLO_NSv_as (double as, int i) {return ( as*gammaNSv_LO(i) ) / ( -beta0*as*as );}
  inline complex <double> gammaNLO_NSp_as (double as, int i) {return ( as*gammaNSp_LO(i) + as*as*gammaNSp_NLO(i) ) / ( -beta0*as*as -beta1*as*as*as);}
  inline complex <double> gammaNLO_NSm_as (double as, int i) {return ( as*gammaNSm_LO(i) + as*as*gammaNSm_NLO(i) ) / ( -beta0*as*as -beta1*as*as*as);}
  inline complex <double> gammaNLO_NSv_as (double as, int i) {return ( as*gammaNSv_LO(i) + as*as*gammaNSv_NLO(i) ) / ( -beta0*as*as -beta1*as*as*as);}
  inline complex <double> gammaNNLO_NSp_as (double as, int i) {return ( as*gammaNSp_LO(i) + as*as*gammaNSp_NLO(i) + as*as*as*gammaNSp_NNLO(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as);}
  inline complex <double> gammaNNLO_NSm_as (double as, int i) {return ( as*gammaNSm_LO(i) + as*as*gammaNSm_NLO(i) + as*as*as*gammaNSm_NNLO(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as);}
  inline complex <double> gammaNNLO_NSv_as (double as, int i) {return ( as*gammaNSv_LO(i) + as*as*gammaNSv_NLO(i) + as*as*as*gammaNSv_NNLO(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as);}
  inline complex <double> gammaNNNLO_NSp_as (double as, int i) {return ( as*gammaNSp_LO(i) + as*as*gammaNSp_NLO(i) + as*as*as*gammaNSp_NNLO(i) + as*as*as*as*gammaNSp_NNNLO(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as -beta3*as*as*as*as*as);}
  inline complex <double> gammaNNNLO_NSm_as (double as, int i) {return ( as*gammaNSm_LO(i) + as*as*gammaNSm_NLO(i) + as*as*as*gammaNSm_NNLO(i) + as*as*as*as*gammaNSm_NNNLO(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as -beta3*as*as*as*as*as);}
  inline complex <double> gammaNNNLO_NSv_as (double as, int i) {return ( as*gammaNSv_LO(i) + as*as*gammaNSv_NLO(i) + as*as*as*gammaNSv_NNLO(i) + as*as*as*as*gammaNSv_NNNLO(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as -beta3*as*as*as*as*as);}
  inline complex <double> gammaLO_NSp_as_1 (double as, int i) {return ( as*gammaNSp_LO_1(i) ) / ( -beta0*as*as );}
  inline complex <double> gammaLO_NSm_as_1 (double as, int i) {return ( as*gammaNSm_LO_1(i) ) / ( -beta0*as*as );}
  inline complex <double> gammaLO_NSv_as_1 (double as, int i) {return ( as*gammaNSv_LO_1(i) ) / ( -beta0*as*as );}
  inline complex <double> gammaNLO_NSp_as_1 (double as, int i) {return ( as*gammaNSp_LO_1(i) + as*as*gammaNSp_NLO_1(i) ) / ( -beta0*as*as -beta1*as*as*as);}
  inline complex <double> gammaNLO_NSm_as_1 (double as, int i) {return ( as*gammaNSm_LO_1(i) + as*as*gammaNSm_NLO_1(i) ) / ( -beta0*as*as -beta1*as*as*as);}
  inline complex <double> gammaNLO_NSv_as_1 (double as, int i) {return ( as*gammaNSv_LO_1(i) + as*as*gammaNSv_NLO_1(i) ) / ( -beta0*as*as -beta1*as*as*as);}
  inline complex <double> gammaNNLO_NSp_as_1 (double as, int i) {return ( as*gammaNSp_LO_1(i) + as*as*gammaNSp_NLO_1(i) + as*as*as*gammaNSp_NNLO_1(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as);}
  inline complex <double> gammaNNLO_NSm_as_1 (double as, int i) {return ( as*gammaNSm_LO_1(i) + as*as*gammaNSm_NLO_1(i) + as*as*as*gammaNSm_NNLO_1(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as);}
  inline complex <double> gammaNNLO_NSv_as_1 (double as, int i) {return ( as*gammaNSv_LO_1(i) + as*as*gammaNSv_NLO_1(i) + as*as*as*gammaNSv_NNLO_1(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as);}
  inline complex <double> gammaNNNLO_NSp_as_1 (double as, int i) {return ( as*gammaNSp_LO_1(i) + as*as*gammaNSp_NLO_1(i) + as*as*as*gammaNSp_NNLO_1(i) + as*as*as*as*gammaNSp_NNNLO_1(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as -beta3*as*as*as*as*as);}
  inline complex <double> gammaNNNLO_NSm_as_1 (double as, int i) {return ( as*gammaNSm_LO_1(i) + as*as*gammaNSm_NLO_1(i) + as*as*as*gammaNSm_NNLO_1(i) + as*as*as*as*gammaNSm_NNNLO_1(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as -beta3*as*as*as*as*as);}
  inline complex <double> gammaNNNLO_NSv_as_1 (double as, int i) {return ( as*gammaNSv_LO_1(i) + as*as*gammaNSv_NLO_1(i) + as*as*as*gammaNSv_NNLO_1(i) + as*as*as*as*gammaNSv_NNNLO_1(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as -beta3*as*as*as*as*as);}
  inline complex <double> gammaLO_NSp_as_2 (double as, int i) {return ( as*gammaNSp_LO_2(i) ) / ( -beta0*as*as );}
  inline complex <double> gammaLO_NSm_as_2 (double as, int i) {return ( as*gammaNSm_LO_2(i) ) / ( -beta0*as*as );}
  inline complex <double> gammaLO_NSv_as_2 (double as, int i) {return ( as*gammaNSv_LO_2(i) ) / ( -beta0*as*as );}
  inline complex <double> gammaNLO_NSp_as_2 (double as, int i) {return ( as*gammaNSp_LO_2(i) + as*as*gammaNSp_NLO_2(i) ) / ( -beta0*as*as -beta1*as*as*as);}
  inline complex <double> gammaNLO_NSm_as_2 (double as, int i) {return ( as*gammaNSm_LO_2(i) + as*as*gammaNSm_NLO_2(i) ) / ( -beta0*as*as -beta1*as*as*as);}
  inline complex <double> gammaNLO_NSv_as_2 (double as, int i) {return ( as*gammaNSv_LO_2(i) + as*as*gammaNSv_NLO_2(i) ) / ( -beta0*as*as -beta1*as*as*as);}
  inline complex <double> gammaNNLO_NSp_as_2 (double as, int i) {return ( as*gammaNSp_LO_2(i) + as*as*gammaNSp_NLO_2(i) + as*as*as*gammaNSp_NNLO_2(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as);}
  inline complex <double> gammaNNLO_NSm_as_2 (double as, int i) {return ( as*gammaNSm_LO_2(i) + as*as*gammaNSm_NLO_2(i) + as*as*as*gammaNSm_NNLO_2(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as);}
  inline complex <double> gammaNNLO_NSv_as_2 (double as, int i) {return ( as*gammaNSv_LO_2(i) + as*as*gammaNSv_NLO_2(i) + as*as*as*gammaNSv_NNLO_2(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as);}
  inline complex <double> gammaNNNLO_NSp_as_2 (double as, int i) {return ( as*gammaNSp_LO_2(i) + as*as*gammaNSp_NLO_2(i) + as*as*as*gammaNSp_NNLO_2(i) + as*as*as*as*gammaNSp_NNNLO_2(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as -beta3*as*as*as*as*as);}
  inline complex <double> gammaNNNLO_NSm_as_2 (double as, int i) {return ( as*gammaNSm_LO_2(i) + as*as*gammaNSm_NLO_2(i) + as*as*as*gammaNSm_NNLO_2(i) + as*as*as*as*gammaNSm_NNNLO_2(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as -beta3*as*as*as*as*as);}
  inline complex <double> gammaNNNLO_NSv_as_2 (double as, int i) {return ( as*gammaNSv_LO_2(i) + as*as*gammaNSv_NLO_2(i) + as*as*as*gammaNSv_NNLO_2(i) + as*as*as*as*gammaNSv_NNNLO_2(i) ) / ( -beta0*as*as -beta1*as*as*as -beta2*as*as*as*as -beta3*as*as*as*as*as);}

  void init();
  void release();
  void allocate();
  void free();
  void update();
  void update_pdf();
  void retrieve_pdf_mub();
  void retrieve_pdf_muf();
  void update_RU();
  void evolve(double asi, complex <double> asf, int order, bool hfswitch = false, int hfl = 0, bool smallx = false);
  void evolve();
  void commute();
}

#endif
